@@ -88,22 +88,22 @@ static GParamSpec *props[PROP_LAST_PROP];
 
 
 typedef struct _PhocDesktopPrivate {
-  PhocIdleInhibit       *idle_inhibit;
+  PhocIdleInhibit   *idle_inhibit;
 
-  gboolean               enable_animations;
+  gboolean           enable_animations;
 
-  GSettings             *settings;
-  GSettings             *interface_settings;
+  GSettings         *settings;
+  GSettings         *interface_settings;
 
-  PhocOutputsStates     *outputs_states;
+  PhocOutputsStates *outputs_states;
 
   /* Protocols from wlroots */
   struct wlr_data_control_manager_v1 *data_control_manager_v1;
   struct wlr_ext_image_copy_capture_manager_v1 *ext_image_copy_capture_manager_v1;
-  struct wlr_idle_notifier_v1 *idle_notifier_v1;
-  struct wlr_screencopy_manager_v1 *screencopy_manager_v1;
-  struct wl_listener gamma_control_set_gamma;
-  struct wl_listener request_set_cursor_shape;
+  struct wlr_idle_notifier_v1        *idle_notifier_v1;
+  struct wlr_screencopy_manager_v1   *screencopy_manager_v1;
+  struct wl_listener     gamma_control_set_gamma;
+  struct wl_listener     request_set_cursor_shape;
 
   /* Protocols without upstreamable implementations */
   PhocPhoshPrivate      *phosh;
@@ -111,7 +111,6 @@ typedef struct _PhocDesktopPrivate {
 
   /* Protocols that should go upstream */
   PhocLayerShellEffects *layer_shell_effects;
-  PhocDeviceState       *device_state;
 
   PhocWorkspaceManager  *workspace_manager;
   PhocWorkspace         *active_workspace;
@@ -508,7 +507,7 @@ handle_pointer_constraint (struct wl_listener *listener, void *data)
 
 static void
 auto_maximize_changed_cb (PhocDesktop *self,
-                          const gchar *key,
+                          const char  *key,
                           GSettings   *settings)
 {
   gboolean max = g_settings_get_boolean (settings, key);
@@ -522,7 +521,7 @@ auto_maximize_changed_cb (PhocDesktop *self,
 
 static void
 on_enable_animations_changed (PhocDesktop *self,
-                              const gchar *key,
+                              const char  *key,
                               GSettings   *settings)
 {
   PhocDesktopPrivate *priv;
@@ -785,7 +784,6 @@ phoc_desktop_finalize (GObject *object)
   g_clear_object (&priv->phosh);
   g_clear_pointer (&priv->gtk_shell, phoc_gtk_shell_destroy);
   g_clear_object (&priv->layer_shell_effects);
-  g_clear_object (&priv->device_state);
   g_clear_pointer (&self->layout, wlr_output_layout_destroy);
 
   g_clear_object (&priv->outputs_states);
@@ -1635,7 +1633,7 @@ phoc_desktop_get_saved_outputs_state (PhocDesktop *self, const char *output_iden
 /**
  * phoc_desktop_get_workspace_manager:
  * @self: the desktop
-
+ *
  * Get the workspace manager
  *
  * Returns:(transfer none): The workspace manager
@@ -1653,7 +1651,7 @@ phoc_desktop_get_workspace_manager (PhocDesktop *self)
 /**
  * phoc_desktop_get_active_workspace:
  * @self: the desktop
-
+ *
  * Get the workspace manager
  *
  * Returns:(transfer none): The workspace manager
