@@ -14,6 +14,7 @@
  */
 #include "phoc-config.h"
 #include "keybindings.h"
+#include "workspace-manager.h"
 #include "seat.h"
 #include "server.h"
 #include "keyboard.h"
@@ -35,7 +36,7 @@ typedef void (*PhocKeyHandlerFunc) (PhocSeat *seat, GVariant *param);
  */
 typedef struct
 {
-  gchar              *name;
+  char               *name;
   PhocKeyHandlerFunc  func;
   GVariant           *param;
 
@@ -209,9 +210,109 @@ handle_switch_input_source (PhocSeat *seat, GVariant *param)
 }
 
 
+static void
+handle_switch_to_workspace (PhocSeat *seat, GVariant *param)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  int index = g_variant_get_int32 (param);
+
+  phoc_workspace_manager_set_active_by_index (manager, index);
+}
+
+
+static void
+handle_switch_to_workspace_relative (PhocSeat *seat, GVariant *param)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  int index = phoc_workspace_manager_get_active_index (manager);
+  int offset = g_variant_get_int32 (param);
+
+  phoc_workspace_manager_set_active_by_index (manager, index + offset);
+}
+
+
+static void
+move_to_workspace (PhocView *view, int index)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  guint n_workspaces = phoc_workspace_manager_get_n_workspaces (manager);
+  PhocWorkspace *active, *new;
+
+  g_assert (index >= 0);
+  g_assert (index < n_workspaces);
+
+  active = phoc_workspace_manager_get_active (manager);
+  new = phoc_workspace_manager_get_by_index (manager, index);
+  if (active == new)
+    return;
+
+  phoc_view_damage_whole (view);
+  if (!phoc_workspace_remove_view (active, view)) {
+    g_critical ("Current active view '%s' not found on active workspace %d",
+                phoc_view_get_app_id (view),
+                phoc_workspace_manager_get_active_index (manager));
+    return;
+  }
+
+  phoc_workspace_insert_view (new, view);
+  /* Switch to the workspace containing the moved view */
+  phoc_workspace_manager_set_active_by_index (manager, index);
+}
+
+
+static void
+handle_move_to_workspace (PhocSeat *seat, GVariant *param)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  PhocView *view = phoc_seat_get_focus_view (seat);
+  int index = g_variant_get_int32 (param);
+  guint n_workspaces = phoc_workspace_manager_get_n_workspaces (manager);
+
+  if (!view)
+    return;
+
+  if (index < 0)
+    return;
+
+  if (index >= n_workspaces)
+    return;
+
+  move_to_workspace (view, index);
+}
+
+
+static void
+handle_move_to_workspace_relative (PhocSeat *seat, GVariant *param)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocWorkspaceManager *manager = phoc_desktop_get_workspace_manager (desktop);
+  PhocView *view = phoc_seat_get_focus_view (seat);
+  int offset = g_variant_get_int32 (param);
+  int index = phoc_workspace_manager_get_active_index (manager);
+  guint n_workspaces = phoc_workspace_manager_get_n_workspaces (manager);
+
+  if (!view)
+    return;
+
+  index += offset;
+
+  if (index < 0)
+    return;
+
+  if (index >= n_workspaces)
+    return;
+
+  move_to_workspace (view, index);
+}
+
+
 /* This is copied from mutter which in turn got it form GTK+ */
 static inline gboolean
-is_alt (const gchar *string)
+is_alt (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'a' || string[1] == 'A') &&
@@ -221,7 +322,7 @@ is_alt (const gchar *string)
 }
 
 static inline gboolean
-is_ctl (const gchar *string)
+is_ctl (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'c' || string[1] == 'C') &&
@@ -231,7 +332,7 @@ is_ctl (const gchar *string)
 }
 
 static inline gboolean
-is_modx (const gchar *string)
+is_modx (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'm' || string[1] == 'M') &&
@@ -242,7 +343,7 @@ is_modx (const gchar *string)
 }
 
 static inline gboolean
-is_ctrl (const gchar *string)
+is_ctrl (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'c' || string[1] == 'C') &&
@@ -253,7 +354,7 @@ is_ctrl (const gchar *string)
 }
 
 static inline gboolean
-is_shft (const gchar *string)
+is_shft (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 's' || string[1] == 'S') &&
@@ -264,7 +365,7 @@ is_shft (const gchar *string)
 }
 
 static inline gboolean
-is_shift (const gchar *string)
+is_shift (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 's' || string[1] == 'S') &&
@@ -276,7 +377,7 @@ is_shift (const gchar *string)
 }
 
 static inline gboolean
-is_control (const gchar *string)
+is_control (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'c' || string[1] == 'C') &&
@@ -290,7 +391,7 @@ is_control (const gchar *string)
 }
 
 static inline gboolean
-is_meta (const gchar *string)
+is_meta (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'm' || string[1] == 'M') &&
@@ -301,7 +402,7 @@ is_meta (const gchar *string)
 }
 
 static inline gboolean
-is_super (const gchar *string)
+is_super (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 's' || string[1] == 'S') &&
@@ -313,7 +414,7 @@ is_super (const gchar *string)
 }
 
 static inline gboolean
-is_hyper (const gchar *string)
+is_hyper (const char *string)
 {
   return ((string[0] == '<') &&
           (string[1] == 'h' || string[1] == 'H') &&
@@ -325,7 +426,7 @@ is_hyper (const gchar *string)
 }
 
 static inline gboolean
-is_keycode (const gchar *string)
+is_keycode (const char *string)
 {
   return (string[0] == '0' &&
           string[1] == 'x' &&
@@ -340,7 +441,7 @@ is_keycode (const gchar *string)
  * and symbols.
  */
 PhocKeyCombo *
-phoc_parse_accelerator (const gchar *accelerator)
+phoc_parse_accelerator (const char *accelerator)
 {
   PhocKeyCombo *combo;
   xkb_keysym_t keyval;
@@ -403,7 +504,7 @@ phoc_parse_accelerator (const gchar *accelerator)
         len -= 7;
         mods |= WLR_MODIFIER_LOGO;
       } else {
-        gchar last_ch;
+        char last_ch;
 
         last_ch = *accelerator;
         while (last_ch && last_ch != '>') {
@@ -423,7 +524,7 @@ phoc_parse_accelerator (const gchar *accelerator)
       } else {
         keyval = xkb_keysym_from_name (accelerator, XKB_KEYSYM_CASE_INSENSITIVE);
         if (keyval == XKB_KEY_NoSymbol) {
-          g_autofree gchar *with_xf86 = g_strconcat ("XF86", accelerator, NULL);
+          g_autofree char *with_xf86 = g_strconcat ("XF86", accelerator, NULL);
           keyval = xkb_keysym_from_name (with_xf86, XKB_KEYSYM_CASE_INSENSITIVE);
 
           if (keyval == XKB_KEY_NoSymbol)
@@ -463,7 +564,7 @@ key_combo_eq (const PhocKeyCombo *sym1, const PhocKeyCombo *sym2)
 
 
 static gboolean
-keybinding_by_name (const PhocKeybinding *keybinding, const gchar *name)
+keybinding_by_name (const PhocKeybinding *keybinding, const char *name)
 {
   return g_strcmp0 (keybinding->name, name);
 }
@@ -483,7 +584,7 @@ keybinding_by_key_combo (const PhocKeybinding *keybinding, const PhocKeyCombo *c
 
 static void
 on_keybinding_setting_changed (PhocKeybindings *self,
-                               const gchar     *key,
+                               const char      *key,
                                GSettings       *settings)
 {
   g_auto(GStrv) accelerators = NULL;
@@ -523,11 +624,11 @@ on_keybinding_setting_changed (PhocKeybindings *self,
 static gboolean
 phoc_add_keybinding (PhocKeybindings    *self,
                      GSettings          *settings,
-                     const gchar        *name,
+                     const char         *name,
                      PhocKeyHandlerFunc  func,
                      GVariant           *param)
 {
-  g_autofree gchar *signal_name = NULL;
+  g_autofree char *signal_name = NULL;
   PhocKeybinding *binding;
 
   if (g_slist_find_custom (self->bindings, name, (GCompareFunc)keybinding_by_name)) {
@@ -627,6 +728,48 @@ phoc_keybindings_constructed (GObject *object)
   phoc_add_keybinding (self, self->settings,
                        "switch-input-source", handle_switch_input_source,
                        NULL);
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-1", handle_switch_to_workspace,
+                       g_variant_new_int32 (0));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-2", handle_switch_to_workspace,
+                       g_variant_new_int32 (1));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-3", handle_switch_to_workspace,
+                       g_variant_new_int32 (2));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-4", handle_switch_to_workspace,
+                       g_variant_new_int32 (3));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-5", handle_switch_to_workspace,
+                       g_variant_new_int32 (4));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-6", handle_switch_to_workspace,
+                       g_variant_new_int32 (5));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-left", handle_switch_to_workspace_relative,
+                       g_variant_new_int32 (-1));
+  phoc_add_keybinding (self, self->settings,
+                       "switch-to-workspace-right", handle_switch_to_workspace_relative,
+                       g_variant_new_int32 (+1));
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-workspace-1", handle_move_to_workspace,
+                       g_variant_new_int32 (0));
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-workspace-2", handle_move_to_workspace,
+                       g_variant_new_int32 (1));
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-workspace-3", handle_move_to_workspace,
+                       g_variant_new_int32 (2));
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-workspace-4", handle_move_to_workspace,
+                       g_variant_new_int32 (3));
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-workspace-left", handle_move_to_workspace_relative,
+                       g_variant_new_int32 (-1));
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-workspace-right", handle_move_to_workspace_relative,
+                       g_variant_new_int32 (+1));
 
   self->mutter_settings = g_settings_new (MUTTER_KEYBINDINGS_SCHEMA_ID);
   phoc_add_keybinding (self, self->mutter_settings,
