@@ -1,4 +1,3 @@
-
 #define G_LOG_DOMAIN "phoc-output"
 
 #include "phoc-config.h"
@@ -22,7 +21,7 @@
 #include "anim/animatable.h"
 #include "bling.h"
 #include "cursor.h"
-#include "cutouts-overlay.h"
+#include "output-cutouts.h"
 #include "settings.h"
 #include "layer-shell.h"
 #include "layer-shell-effects.h"
@@ -53,35 +52,34 @@ enum {
 static guint signals[N_SIGNALS];
 
 typedef struct _PhocOutputPrivate {
-  PhocRenderer            *renderer;
-  PhocOutputShield        *shield;
+  PhocOutputShield *shield;
 
-  GSList                  *frame_callbacks; /* (element-type: PhocOutputFrameCallbackInfo) */
-  gint                     frame_callback_next_id;
-  gint64                   last_frame_us;
+  GSList  *frame_callbacks; /* (element-type: PhocOutputFrameCallbackInfo) */
+  gint     frame_callback_next_id;
+  gint64   last_frame_us;
 
-  PhocCutoutsOverlay      *cutouts;
-  gulong                   render_cutouts_id;
-  struct wlr_texture      *cutouts_texture;
+  PhocOutputCutouts *cutouts;
+  gulong   render_cutouts_id;
+  struct wlr_texture *cutouts_texture;
 
   gboolean shell_revealed;
   gboolean force_shell_reveal;
 
-  struct wl_listener     damage;
-  struct wl_listener     frame;
-  struct wl_listener     needs_frame;
-  struct wl_listener     request_state;
+  struct wl_listener    damage;
+  struct wl_listener    frame;
+  struct wl_listener    needs_frame;
+  struct wl_listener    request_state;
 
-  PhocOutputScaleFilter  scale_filter;
-  gboolean               gamma_lut_changed;
+  PhocOutputScaleFilter scale_filter;
+  gboolean gamma_lut_changed;
 
-  GQueue                *layer_surfaces[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY + 1];
+  GQueue  *layer_surfaces[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY + 1];
 
   PhocLayoutTransaction *transaction;
-  gboolean               modeset_shield;
+  gboolean modeset_shield;
 
-  GSList                *blings; /* (element-type: PhocBling) */
-  GSList                *debug_damage; /* (element-type: PhocDebugDamageRegion) */
+  GSList  *blings;          /* (element-type: PhocBling) */
+  GSList  *debug_damage;    /* (element-type: PhocDebugDamageRegion) */
 } PhocOutputPrivate;
 
 static void phoc_output_initable_iface_init (GInitableIface *iface);
@@ -293,8 +291,7 @@ phoc_output_animatable_interface_init (PhocAnimatableInterface *iface)
 static void
 phoc_output_init (PhocOutput *self)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocOutputPrivate *priv = phoc_output_get_instance_private(self);
+  PhocOutputPrivate *priv = phoc_output_get_instance_private (self);
 
   priv->frame_callback_next_id = 1;
   priv->last_frame_us = g_get_monotonic_time ();
@@ -311,8 +308,6 @@ phoc_output_init (PhocOutput *self)
 
   priv->scale_filter = PHOC_OUTPUT_SCALE_FILTER_AUTO;
 
-  priv->renderer = g_object_ref (phoc_server_get_renderer (server));
-
   g_signal_connect_object (phoc_layout_transaction_get_default (),
                            "notify::active",
                            G_CALLBACK (on_transaction_active_changed),
@@ -560,12 +555,11 @@ build_debug_damage_tracking (PhocOutput *self)
       priv->debug_damage = g_slist_delete_link (priv->debug_damage, elem);
     }
     elem = next;
-  };
-
-  if (pixman_region32_not_empty (&highlight_damage)) {
-    wlr_damage_ring_add (&self->damage_ring, &highlight_damage);
   }
 
+  if (pixman_region32_not_empty (&highlight_damage))
+    wlr_damage_ring_add (&self->damage_ring, &highlight_damage);
+
   pixman_region32_fini (&highlight_damage);
 }
 
@@ -573,10 +567,11 @@ build_debug_damage_tracking (PhocOutput *self)
 PHOC_TRACE_NO_INLINE static void
 phoc_output_draw (PhocOutput *self)
 {
+  PhocServer *server = phoc_server_get_default ();
   PhocOutputPrivate *priv = phoc_output_get_instance_private (self);
   struct wlr_output *wlr_output = self->wlr_output;
   bool needs_frame, scanned_out = false;
-  pixman_region32_t buffer_damage, frame_damage;
+  pixman_region32_t buffer_damage;
   PhocRenderContext render_context;
   struct wlr_buffer *buffer;
   struct wlr_render_pass *render_pass;
@@ -589,7 +584,6 @@ phoc_output_draw (PhocOutput *self)
   needs_frame = wlr_output->needs_frame;
   needs_frame |= pixman_region32_not_empty (&self->damage_ring.current);
   needs_frame |= priv->gamma_lut_changed;
-  needs_frame |= (priv->debug_damage != NULL);
 
   if (!needs_frame)
     return;
@@ -597,10 +591,7 @@ phoc_output_draw (PhocOutput *self)
   if (G_UNLIKELY (priv->gamma_lut_changed))
     phoc_output_set_gamma_lut (self, &pending);
 
-  pixman_region32_init (&frame_damage);
-  pixman_region32_copy (&frame_damage, &self->damage_ring.current);
-  wlr_output_state_set_damage (&pending, &frame_damage);
-  pixman_region32_fini (&frame_damage);
+  wlr_output_state_set_damage (&pending, &self->damage_ring.current);
 
   /* Check if we can delegate the fullscreen surface to the output */
   if (self->fullscreen_view)
@@ -631,7 +622,7 @@ phoc_output_draw (PhocOutput *self)
     .alpha = 1.0,
     .render_pass = render_pass,
   };
-  phoc_renderer_render_output (priv->renderer, self, &render_context);
+  phoc_renderer_render_output (phoc_server_get_renderer (server), self, &render_context);
 
   pixman_region32_fini (&buffer_damage);
 
@@ -651,7 +642,7 @@ phoc_output_draw (PhocOutput *self)
  out:
   wlr_output_state_finish (&pending);
 
-  flags = phoc_server_get_debug_flags (phoc_server_get_default ());
+  flags = phoc_server_get_debug_flags (server);
   if (G_UNLIKELY (flags & PHOC_SERVER_DEBUG_FLAG_DAMAGE_WHOLE))
     phoc_output_damage_whole (self);
 }
@@ -671,7 +662,7 @@ phoc_output_handle_frame (struct wl_listener *listener, void *data)
     PhocOutputFrameCallbackInfo *cb_info = l->data;
     gboolean ret;
 
-    ret = cb_info->callback(cb_info->animatable, priv->last_frame_us, cb_info->user_data);
+    ret = cb_info->callback (cb_info->animatable, priv->last_frame_us, cb_info->user_data);
     if (ret == G_SOURCE_REMOVE) {
       phoc_output_frame_callback_info_free (cb_info);
       priv->frame_callbacks = g_slist_delete_link (priv->frame_callbacks, l);
@@ -749,6 +740,7 @@ phoc_output_handle_commit (struct wl_listener *listener, void *data)
       phoc_output_raise_shield (self, FALSE);
       priv->modeset_shield = TRUE;
     }
+    phoc_output_damage_whole (self);
   }
 
   if (event->state->committed & (WLR_OUTPUT_STATE_ENABLED |
@@ -758,11 +750,6 @@ phoc_output_handle_commit (struct wl_listener *listener, void *data)
     update_output_manager_config (self->desktop);
   }
 
-  if (event->state->committed & (WLR_OUTPUT_STATE_MODE |
-                                 WLR_OUTPUT_STATE_TRANSFORM)) {
-    wlr_output_schedule_frame (self->wlr_output);
-  }
-
   if (event->state->committed & WLR_OUTPUT_STATE_ENABLED && self->wlr_output->enabled) {
     priv->gamma_lut_changed = TRUE;
     wlr_output_schedule_frame (self->wlr_output);
@@ -1010,7 +997,6 @@ phoc_output_initable_init (GInitable    *initable,
   PhocOutputPrivate *priv = phoc_output_get_instance_private (self);
   PhocOutputConfig *output_config;
   struct wlr_output_state pending;
-  int width, height;
 
   self->wlr_output->data = self;
   wl_list_insert (&self->desktop->outputs, &self->link);
@@ -1025,6 +1011,7 @@ phoc_output_initable_init (GInitable    *initable,
   }
 
   wlr_damage_ring_init (&self->damage_ring);
+  phoc_output_damage_whole (self);
 
   self->output_destroy.notify = phoc_output_handle_destroy;
   wl_signal_add (&self->wlr_output->events.destroy, &self->output_destroy);
@@ -1060,6 +1047,7 @@ phoc_output_initable_init (GInitable    *initable,
   phoc_output_fill_state (self, output_config, &pending);
   phoc_output_set_layout_pos (self, output_config);
   wlr_output_commit_state (self->wlr_output, &pending);
+  wlr_output_state_finish (&pending);
 
   for (GSList *elem = phoc_input_get_seats (input); elem; elem = elem->next) {
     PhocSeat *seat = PHOC_SEAT (elem->data);
@@ -1071,17 +1059,15 @@ phoc_output_initable_init (GInitable    *initable,
 
   phoc_layer_shell_arrange (self);
   phoc_layer_shell_update_focus ();
-  phoc_output_damage_whole (self);
 
   update_output_manager_config (self->desktop);
 
-  wlr_output_transformed_resolution (self->wlr_output, &width, &height);
-
-  if (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_CUTOUTS)) {
-    priv->cutouts = phoc_cutouts_overlay_new (phoc_server_get_compatibles (server));
+  if (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_CUTOUTS) &&
+      phoc_output_is_builtin (self)) {
+    priv->cutouts = phoc_output_cutouts_new (phoc_server_get_compatibles (server));
     if (priv->cutouts) {
       g_message ("Adding cutouts overlay");
-      priv->cutouts_texture = phoc_cutouts_overlay_get_cutouts_texture (priv->cutouts, self);
+      priv->cutouts_texture = phoc_output_cutouts_get_cutouts_texture (priv->cutouts);
       priv->render_cutouts_id = g_signal_connect_swapped (renderer, "render-end",
                                                           G_CALLBACK (render_cutouts),
                                                           self);
@@ -1090,8 +1076,6 @@ phoc_output_initable_init (GInitable    *initable,
     }
   }
 
-  wlr_output_state_finish (&pending);
-
   g_message ("Output '%s' added ('%s'/'%s'/'%s'), "
              "%" PRId32 "mm x %" PRId32 "mm",
              self->wlr_output->name,
@@ -1107,8 +1091,10 @@ phoc_output_initable_init (GInitable    *initable,
 static void
 phoc_output_finalize (GObject *object)
 {
+  PhocServer *server = phoc_server_get_default ();
   PhocOutput *self = PHOC_OUTPUT (object);
   PhocOutputPrivate *priv = phoc_output_get_instance_private (self);
+  PhocRenderer *renderer = phoc_server_get_renderer (server);
 
   self->wlr_output->data = NULL;
   self->wlr_output = NULL;
@@ -1127,10 +1113,11 @@ phoc_output_finalize (GObject *object)
   for (int i = 0; i < G_N_ELEMENTS (priv->layer_surfaces); i++)
     g_clear_pointer (&priv->layer_surfaces[i], g_queue_free);
 
-  g_clear_signal_handler (&priv->render_cutouts_id, priv->renderer);
-  g_clear_object (&priv->renderer);
+  if (renderer) {
+    g_clear_signal_handler (&priv->render_cutouts_id, renderer);
+    g_clear_pointer (&priv->cutouts_texture, wlr_texture_destroy);
+  }
   g_clear_object (&priv->cutouts);
-  g_clear_pointer (&priv->cutouts_texture, wlr_texture_destroy);
   g_clear_object (&priv->shield);
   g_clear_object (&self->desktop);
 
@@ -1620,10 +1607,17 @@ phoc_output_for_each_surface (PhocOutput          *self,
 void
 phoc_output_damage_whole (PhocOutput *self)
 {
+  pixman_region32_t damage;
+  int width, height;
+
   if (self == NULL || self->wlr_output == NULL)
     return;
 
-  wlr_damage_ring_add_whole (&self->damage_ring);
+  wlr_output_transformed_resolution (self->wlr_output, &width, &height);
+  pixman_region32_init_rect (&damage, 0, 0, width, height);
+  phoc_output_damage_region (self, &damage);
+  pixman_region32_fini (&damage);
+
   wlr_output_schedule_frame (self->wlr_output);
 }
 
@@ -1729,9 +1723,8 @@ damage_view_blings (PhocOutput *self, PhocView  *view)
 void
 phoc_output_damage_from_view (PhocOutput *self, PhocView *view, bool whole)
 {
-  if (!phoc_view_accept_damage (self, view)) {
+  if (!phoc_view_accept_damage (self, view))
     return;
-  }
 
   if (whole)
     damage_view_blings (self, view);
@@ -1821,7 +1814,7 @@ phoc_output_damage_region (PhocOutput *self, const pixman_region32_t *region)
   pixman_region32_init (&clipped);
   pixman_region32_intersect_rect (&clipped, region, 0, 0, width, height);
 
-  if (!pixman_region32_not_empty (&clipped)) {
+  if (pixman_region32_empty (&clipped)) {
     pixman_region32_fini (&clipped);
     return FALSE;
   }
@@ -2058,21 +2051,34 @@ phoc_output_handle_output_power_manager_set_mode (struct wl_listener *listener,
  * phone LCD), %FALSE otherwise.
  */
 gboolean
-phoc_output_is_builtin (PhocOutput *output)
+phoc_output_is_builtin (PhocOutput *self)
 {
-  g_return_val_if_fail (output, FALSE);
-  g_return_val_if_fail (output->wlr_output, FALSE);
-  g_return_val_if_fail (output->wlr_output->name, FALSE);
+  PhocServer *server = phoc_server_get_default ();
+  const char *name;
 
-  if (g_str_has_prefix (output->wlr_output->name, "LVDS-"))
+  g_return_val_if_fail (self, FALSE);
+  g_return_val_if_fail (self->wlr_output, FALSE);
+  g_return_val_if_fail (self->wlr_output->name, FALSE);
+
+  name = self->wlr_output->name;
+  if (g_str_has_prefix (name, "LVDS-"))
     return TRUE;
-  else if (g_str_has_prefix (output->wlr_output->name, "eDP-"))
+  else if (g_str_has_prefix (name, "eDP-"))
     return TRUE;
-  else if (g_str_has_prefix (output->wlr_output->name, "DSI-"))
+  else if (g_str_has_prefix (name, "DSI-"))
     return TRUE;
-  else if (g_str_has_prefix (output->wlr_output->name, "DPI-"))
+  else if (g_str_has_prefix (name, "DPI-"))
     return TRUE;
 
+  if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_FAKE_BUILTIN))) {
+    if (g_str_has_prefix (name, "WL-"))
+      return TRUE;
+    else if (g_str_has_prefix (name, "X11-"))
+      return TRUE;
+    else if (g_str_has_prefix (name, "HEADLESS-"))
+      return TRUE;
+  }
+
   return FALSE;
 }
 
@@ -2156,7 +2162,7 @@ phoc_output_remove_frame_callback  (PhocOutput *self, guint id)
       return;
     }
   }
-  g_return_if_reached();
+  g_return_if_reached ();
 }
 
 
@@ -2281,7 +2287,7 @@ phoc_output_has_layer (PhocOutput *self, enum zwlr_layer_shell_v1_layer layer)
 static gboolean
 should_reveal_shell (PhocOutput *self)
 {
-  PhocServer *server = phoc_server_get_default();
+  PhocServer *server = phoc_server_get_default ();
   PhocDesktop *desktop = phoc_server_get_desktop (server);
   PhocInput *input = phoc_server_get_input (server);
   PhocOutputPrivate *priv;
@@ -2292,12 +2298,11 @@ should_reveal_shell (PhocOutput *self)
 
   for (GSList *elem = phoc_input_get_seats (input); elem; elem = elem->next) {
     PhocSeat *seat = PHOC_SEAT (elem->data);
-    /* is our layer-surface focused on some seat? */
-    if (seat->focused_layer && seat->focused_layer->output == self->wlr_output) {
+    /* Is our layer-surface focused on some seat? */
+    if (seat->focused_layer && seat->focused_layer->output == self->wlr_output)
       return true;
-    }
 
-    /* is OSK displayed because of our fullscreen view? */
+    /* Is OSK displayed because of our fullscreen view? */
     if (phoc_view_is_mapped (self->fullscreen_view) &&
         phoc_input_method_relay_is_enabled (&seat->im_relay, self->fullscreen_view->wlr_surface)) {
       PhocLayerSurface *osk = phoc_layer_shell_find_osk (self);
@@ -2306,19 +2311,18 @@ should_reveal_shell (PhocOutput *self)
     }
   }
 
-  /* is some draggable surface unfolded, being dragged or animated? */
+  /* Is some draggable surface unfolded, being dragged or animated? */
   wl_list_for_each (layer_surface, &self->layer_surfaces, link) {
     PhocDraggableLayerSurface *draggable;
 
     draggable = phoc_desktop_get_draggable_layer_surface (desktop, layer_surface);
     if (draggable &&
         (phoc_draggable_layer_surface_get_state (draggable) != PHOC_DRAGGABLE_SURFACE_STATE_NONE ||
-         phoc_draggable_layer_surface_is_unfolded (draggable))) {
+         phoc_draggable_layer_surface_is_unfolded (draggable)))
       return true;
-    }
   }
 
-  /* is shell reveal forced by user gesture? */
+  /* Is shell reveal forced by user gesture? */
   return priv->force_shell_reveal;
 }
 
@@ -2343,9 +2347,8 @@ phoc_output_update_shell_reveal (PhocOutput *self)
   old = priv->shell_revealed;
   priv->shell_revealed = should_reveal_shell (self);
 
-  if (priv->shell_revealed != old) {
+  if (priv->shell_revealed != old)
     phoc_output_damage_whole (self);
-  }
 }
 
 /**
@@ -2429,7 +2432,7 @@ phoc_output_handle_gamma_control_set_gamma (struct wl_listener *listener, void *
 {
   const struct wlr_gamma_control_manager_v1_set_gamma_event *event = data;
   PhocOutput *self = PHOC_OUTPUT (event->output->data);
-  PhocOutputPrivate *priv = phoc_output_get_instance_private(self);
+  PhocOutputPrivate *priv = phoc_output_get_instance_private (self);
 
   if (!self)
     return;
@@ -2438,7 +2441,6 @@ phoc_output_handle_gamma_control_set_gamma (struct wl_listener *listener, void *
   wlr_output_schedule_frame (self->wlr_output);
 }
 
-
 /**
  * phoc_output_transform_damage:
  * @self: The output to transform for
@@ -2499,7 +2501,7 @@ phoc_output_get_texture_filter_mode (PhocOutput *self)
     g_assert_not_reached ();
   }
 
-  if (ceilf(self->wlr_output->scale) == self->wlr_output->scale)
+  if (ceilf (self->wlr_output->scale) == self->wlr_output->scale)
     return WLR_SCALE_FILTER_NEAREST;
 
   return WLR_SCALE_FILTER_BILINEAR;
