@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2020 Purism SPC
- *               2025 The Phosh Developers
+ *               2025 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -18,7 +18,7 @@
 struct task_data {
   PhocTestClientFunc   func;
   PhocTestOutputConfig output_config;
-  gpointer             data;
+  gpointer data;
 };
 
 
@@ -195,7 +195,7 @@ output_handle_scale (void *data, struct wl_output *wl_output,
   PhocTestClientGlobals *globals = data;
   PhocTestOutputConfig output_config = globals->output_config;
 
-  g_assert_cmpint(scale, ==, ceil(output_config.scale));
+  g_assert_cmpint (scale, ==, ceil (output_config.scale));
 }
 
 
@@ -225,7 +225,7 @@ foreign_toplevel_handle_title (void                                   *data,
                                const char                             *title)
 {
   PhocTestForeignToplevel *toplevel = data;
-  toplevel->title = g_strdup (title);
+  g_set_str (&toplevel->title, title);
   g_debug ("Got toplevel's title: %p %s", toplevel, title);
 }
 
@@ -520,7 +520,7 @@ build_compositor_config (PhocTestClientIface *iface)
  * otherwise.
  */
 void
-phoc_test_client_run (gint timeout, PhocTestClientIface *iface, gpointer data)
+phoc_test_client_run (int timeout, PhocTestClientIface *iface, gpointer data)
 {
   PhocConfig *config;
   struct task_data td = { .data = data };
@@ -609,7 +609,7 @@ phoc_test_client_create_shm_buffer (PhocTestClientGlobals *globals,
   return TRUE;
 }
 
-static gint
+static int
 foreign_toplevel_compare (gconstpointer data, gconstpointer title)
 {
   const PhocTestForeignToplevel *toplevel = data;
@@ -880,12 +880,13 @@ xdg_toplevel_handle_configure (void *data, struct xdg_toplevel *xdg_toplevel,
 }
 
 static void
-xdg_surface_handle_configure (void *data, struct xdg_surface *xdg_surface,
-                              uint32_t serial)
+xdg_surface_handle_configure (void *data, struct xdg_surface *xdg_surface, uint32_t serial)
 {
   PhocTestXdgToplevelSurface *xs = data;
 
   g_debug ("Configured %p serial %d", xdg_surface, serial);
+  if (xs->ack_configure.callback)
+    (*xs->ack_configure.callback) (xs, xs->ack_configure.data);
   xdg_surface_ack_configure (xs->xdg_surface, serial);
   xs->configured = TRUE;
 }
@@ -907,7 +908,7 @@ static const struct xdg_toplevel_listener xdg_toplevel_listener = {
 
 /**
  * phoc_test_xdg_toplevel_new:
- * @globals: The wayland globals
+ * @globals: The Wayland globals
  * @width: The desired surface width
  * @height: The desired surface height
  * @title: The desired title
@@ -952,14 +953,13 @@ phoc_test_xdg_toplevel_new (PhocTestClientGlobals *globals,
   return xs;
 }
 
-
 /**
- * phoc_test_xdg_toplevel_new:
- * @globals: The wayland globals
+ * phoc_test_xdg_toplevel_new_with_buffer:
+ * @globals: The Wayland globals
  * @width: The desired surface width
  * @height: The desired surface height
  * @title: The desired title
- * @color: Tehe color to fill the surface with
+ * @color: The color to fill the surface with
  *
  * Creates a xdg toplevel with the given property for use in tests.
  * If width and/or height are 0, defaults will be used. Free with
@@ -1004,6 +1004,7 @@ phoc_test_xdg_toplevel_new_with_buffer (PhocTestClientGlobals *globals,
 void
 phoc_test_xdg_toplevel_free (PhocTestXdgToplevelSurface *xs)
 {
+  phoc_test_xdg_toplevel_set_ack_configure_callback (xs, NULL, NULL, NULL);
   xdg_toplevel_destroy (xs->xdg_toplevel);
   xdg_surface_destroy (xs->xdg_surface);
   wl_surface_destroy (xs->wl_surface);
@@ -1012,6 +1013,31 @@ phoc_test_xdg_toplevel_free (PhocTestXdgToplevelSurface *xs)
   g_free (xs);
 }
 
+/**
+ * phoc_test_xdg_toplevel_set_ack_configure_callback:
+ * @xs: The toplevel
+ * @callback: The callback
+ * @data: The data to pass to the callback
+ * @data_free_func: Callback to destroy `data`
+ *
+ * Add a callback to be called before sending the `ack_configure`.
+ * If a callback is already present it will be replaced and the
+ * `destroy_notify` handle invoked to free the data.
+ */
+void
+phoc_test_xdg_toplevel_set_ack_configure_callback (PhocTestXdgToplevelSurface  *xs,
+                                                   PhocXdgAckConfigureCallback  callback,
+                                                   gpointer                     data,
+                                                   GDestroyNotify               data_free_func)
+{
+  if (xs->ack_configure.data_free_func && xs->ack_configure.data)
+    (*xs->ack_configure.data_free_func) (xs->ack_configure.data);
+
+  xs->ack_configure.callback = callback;
+  xs->ack_configure.data = data;
+  xs->ack_configure.data_free_func = data_free_func;
+}
+
 void
 phoc_test_xdg_update_buffer (PhocTestClientGlobals      *globals,
                              PhocTestXdgToplevelSurface *xs,
